@@ -59,6 +59,7 @@ extern "C" {
 #define  CDI_EUNC4                -26   // Unsupported NetCDF4 structure
 #define  CDI_EDIMSIZE             -27   // Invalid dimension size
 #define  CDI_EQENF                -50   // Query entries not found
+#define  CDI_EQNAVAIL             -51   // Query not available for file type
 #define  CDI_ELIMIT               -99   // Internal limits exceeded
 
 // File types
@@ -96,6 +97,7 @@ extern "C" {
 #define  CDI_COMPRESS_AEC           2
 #define  CDI_COMPRESS_ZIP           3
 #define  CDI_COMPRESS_JPEG          4
+#define  CDI_COMPRESS_FILTER        5
 
 // external data types
 
@@ -383,6 +385,8 @@ void    streamDefByteorder(int streamID, int byteorder);
 //      streamInqByteorder: Get the byteorder
 int     streamInqByteorder(int streamID);
 
+void    streamDefFilter(int streamID, int filterId, int nparams, const int *params);
+
 //      streamDefCompType: Define compression type
 void    streamDefCompType(int streamID, int comptype);
 
@@ -431,7 +435,8 @@ void    streamReadVarSlicePart(int streamID, int varID, int levelID, int varType
 void    streamReadVarSlice(int streamID, int varID, int levelID, double data[], SizeType *nmiss);
 void    streamReadVarSliceF(int streamID, int varID, int levelID, float data[], SizeType *nmiss);
 
-void    streamWriteVarChunk(int streamID, int varID, const int rect[3][2], const double data[], SizeType nmiss);
+void    streamWriteVarChunk(int streamID, int varID, const int rect[][2], const double data[], SizeType nmiss);
+void    streamWriteVarChunkF(int streamID, int varID, const int rect[][2], const float data[], SizeType nmiss);
 
 
 // STREAM record I/O routines (sequential access)
@@ -1310,23 +1315,46 @@ extern "C" {
 typedef struct
 {
   int numEntries;
+  // Names
   int numNames;
   bool *namesFound;
   char **names;
+  // Grid cell indices
+  int numCellidx;
+  bool *cellidxFound;
+  size_t *cellidx;
+  // Level indices
+  int numLevidx;
+  bool *levidxFound;
+  int *levidx;
+  // Time step indices
+  int numStepidx;
+  bool *stepidxFound;
+  int *stepidx;
 } CdiQuery;
 
 CdiQuery *cdiQueryCreate();
 CdiQuery *cdiQueryClone(const CdiQuery *query);
 void cdiQueryDelete(CdiQuery *query);
 void cdiQuerySetNames(CdiQuery *query, int numNames, char **names);
+void cdiQuerySetCellidx(CdiQuery *query, int numCellidx, size_t *cellidx);
+void cdiQuerySetLevidx(CdiQuery *query, int numLevidx, int *levidx);
+void cdiQuerySetStepidx(CdiQuery *query, int numStepidx, int *stepidx);
+size_t cdiQueryGetCellidx(const CdiQuery *query, int index);
 int cdiQueryName(CdiQuery *query, const char *name);
+int cdiQueryCellidx(CdiQuery *query, size_t cellidx);
+int cdiQueryLevidx(CdiQuery *query, int levidx);
+int cdiQueryStepidx(CdiQuery *query, int stepidx);
+int cdiQueryNumNames(const CdiQuery *query);
+int cdiQueryNumCellidx(const CdiQuery *query);
+int cdiQueryNumStepidx(const CdiQuery *query);
 int cdiQueryNumEntries(const CdiQuery *query);
 int cdiQueryNumEntriesFound(const CdiQuery *query);
 void cdiQueryPrint(const CdiQuery *query);
 void cdiQueryPrintEntriesNotFound(const CdiQuery *query);
 
 // streamOpenReadQuery: Open a dataset for reading and apply query
-int streamOpenReadQuery(const char *path, const CdiQuery *query);
+int streamOpenReadQuery(const char *path, CdiQuery *query);
 
 // CDI interface for  paraview vtkCDIReader.cxx
 
