@@ -3,6 +3,12 @@
 #include <stdlib.h>
 
 // ==================================================================
+#ifdef __cplusplus
+extern "C"
+{
+#endif
+
+  // clang-format off
 void
 cdiDecodeDate(int date, int *year, int *month, int *day)
 {
@@ -39,6 +45,11 @@ cdiEncodeTime(int hour, int minute, int second)
 {
   return hour * 10000 + minute * 100 + second;
 }
+  // clang-format on
+
+#ifdef __cplusplus
+}
+#endif
 // ==================================================================
 
 CdiDate
@@ -230,11 +241,9 @@ cdiDateTime_isNull(const CdiDateTime cdiDateTime)
   // clang-format on
 }
 
-
 #define DATE_FORMAT "%5.4d-%2.2d-%2.2d"
 #define TIME_FORMAT "%2.2d:%2.2d:%2.2d"
 
-
 const char *
 CdiDateTime_string(CdiDateTime cdiDateTime)
 {
@@ -244,8 +253,7 @@ CdiDateTime_string(CdiDateTime cdiDateTime)
   cdiTime_decode(cdiDateTime.time, &hour, &minute, &second, &ms);
 
   static char datetimeString[64];
-  snprintf(datetimeString, sizeof(datetimeString), DATE_FORMAT "T" TIME_FORMAT,
-           year, month, day, hour, minute, second);
+  snprintf(datetimeString, sizeof(datetimeString), DATE_FORMAT "T" TIME_FORMAT, year, month, day, hour, minute, second);
 
   return datetimeString;
 }
